@@ -35,6 +35,8 @@ from homonim.compare import RasterCompare
 from homonim.enums import ProcCrs, Method
 from homonim.fuse import RasterFuse
 from homonim.kernel_model import KernelModel
+from homonim.stats import ParamStats
+from homonim.errors import ImageFormatError
 
 logger = logging.getLogger(__name__)
 
@@ -74,13 +76,17 @@ def _nodata_cb(ctx, param, value):
     # adapted from rasterio https://github.com/rasterio/rasterio
     if value is None or value.lower() in ["null", "nil", "none", "nada"]:
         return None
-    elif value.lower() == "nan":
-        return float("nan")
     else:
+        # check value is a number and can be cast to output dtype
         try:
-            return float(value)
+            value = float(value.lower())
+            if not rio.dtypes.can_cast_dtype(value, ctx.params['dtype']):
+                raise click.BadParameter(f"{value} cannot be cast to the output image data type {ctx.params['dtype']}",
+                                         param=param, param_hint="nodata")
         except (TypeError, ValueError):
-            raise click.BadParameter("{!r} is not a number".format(value), param=param, param_hint="nodata")
+            raise click.BadParameter(f"{value} is not a number", param=param, param_hint="nodata")
+
+        return value
 
 
 def _creation_options_cb(ctx, param, value):
@@ -115,15 +121,10 @@ def _param_file_cb(ctx, param, value):
     """click callback to validate parameter image file(s)"""
     for filename in value:
         filename = pathlib.Path(filename)
-
-        if not filename.exists():
-            raise click.BadParameter(f'{filename} does not exist')
-
-        with rio.open(filename) as param_im:
-            tags = param_im.tags()
-            if (divmod(param_im.count, 3)[1] != 0 or
-                    not {'HOMO_METHOD', 'HOMO_MODEL_CONF', 'HOMO_PROC_CRS'} <= set(tags)):
-                raise click.BadParameter(f'{filename.name} is not a valid homonim parameter image.', param=param)
+        try:
+            ParamStats._validate_image(filename)
+        except (FileNotFoundError, ImageFormatError):
+            raise click.BadParameter(f'{filename.name} is not a valid parameter image.', param=param)
     return value
 
 
@@ -228,9 +229,9 @@ def cli(verbose, quiet):
 @src_file_arg
 @ref_file_arg
 @click.option("-m", "--method", type=click.Choice(Method, case_sensitive=False),
-              default=Method.gain_blk_offset.name, show_default=True,
+              default=Method.gain_blk_offset,
               help="Homogenisation method.\ngain: Gain-only model. \ngain-blk-offset: Gain-only model applied to "
-                   "offset normalised image blocks. \ngain-offset: Full gain and offset model.")
+                   "offset normalised image blocks [default]. \ngain-offset: Full gain and offset model.")
 @click.option("-k", "--kernel-shape", type=click.Tuple([click.INT, click.INT]), nargs=2, default=(5, 5),
               show_default=True, metavar='<HEIGHT WIDTH>',
               help="Kernel height and width in pixels of the --proc-crs / -pc image.")
@@ -243,10 +244,10 @@ def cli(verbose, quiet):
 @click.option("-nbo", "--no-build-ovw", "build_ovw", type=click.BOOL, is_flag=True, default=True,
               help="Turn off overview building for the homogenised image(s).")
 @click.option("-pc", "--proc-crs", type=click.Choice(ProcCrs, case_sensitive=False),
-              default=ProcCrs.auto.name, show_default=True,
-              help="The image CRS in which to perform parameter estimation.\nauto: estimate in the lowest resolution "
-                   "of the source and reference image CRS's (recommended).\nsrc: estimate in the source image CRS."
-                   "\nref: estimate in the reference image CRS.")
+              default=ProcCrs.auto,
+              help="The image CRS in which to perform parameter estimation.\nauto: Estimate in the lowest resolution "
+                   "of the source and reference image CRS's [default - recommended].\nsrc: Estimate in the source image CRS."
+                   "\nref: Estimate in the reference image CRS.")
 @click.option("-c", "--conf", type=click.Path(exists=True, dir_okay=False, readable=True, path_type=pathlib.Path),
               required=False, default=None, show_default=True,
               help="Path to a yaml configuration file specifying the options below.")
@@ -350,8 +351,7 @@ def fuse(ctx, src_file, ref_file, method, kernel_shape, output_dir, overwrite, d
 
         # compare source and homogenised files with reference (invokes compare command with relevant parameters)
         if do_cmp:
-            ctx.invoke(compare, src_file=compare_files, ref_file=ref_file, proc_crs=proc_crs,
-                       threads=kwargs['threads'])
+            ctx.invoke(compare, src_file=compare_files, ref_file=ref_file, proc_crs=proc_crs)
     except Exception:
         logger.exception("Exception caught during processing")
         raise click.Abort()
@@ -372,7 +372,7 @@ cli.add_command(fuse)
 @output_option
 def compare(src_file, ref_file, proc_crs, output):
     """
-    Report similarity statistics between image(s) with a reference.
+    Report similarity statistics between image(s) and a reference.
 
     INPUTS      Path(s) to image(s) to be compared.
 
@@ -402,6 +402,9 @@ def compare(src_file, ref_file, proc_crs, output):
             res_dict[str(src_filename)] = cmp.compare()
             logger.info(f'Completed in {timer() - start_time:.2f} secs')
 
+        # print a key for the following tables
+        logger.info(f'\n\n{cmp.stats_key}')
+
         # print a results table per source file
         summary_dict = {}
         for src_file, _res_dict in res_dict.items():
@@ -418,7 +421,7 @@ def compare(src_file, ref_file, proc_crs, output):
             summ_df.insert(0, 'File', [pathlib.Path(fn).name for fn in summ_df.index])
             summ_str = summ_df.to_string(float_format="{:.2f}".format, index=False, justify="center",
                                          index_names=False)
-            logger.info(f'\n\nSummary:\n\n{summ_str}')
+            logger.info(f'\n\nSummary over bands:\n\n{summ_str}')
 
         if output is not None:
             res_dict['Reference'] = ref_file.stem
@@ -445,28 +448,31 @@ def stats(param_file, output):
     """
 
     try:
-        cmb_dict = {}
-        # iterate over source files, comparing with reference
+        stats_dict = {}
+        meta_dict = {}
+
+        # process parameter file(s), storing results
         for param_filename in param_file:
-            with rio.open(param_filename, 'r') as param_im:
-                tags = param_im.tags()
-                method = tags['HOMO_METHOD'].replace('_', '-')
-                r2_inpaint_thresh = yaml.safe_load(tags['HOMO_MODEL_CONF'])['r2_inpaint_thresh']
+            logger.info(f'\nProcessing {param_filename.name}')
+            param_stats = ParamStats(param_filename)
+            stats_dict[str(param_filename)] = param_stats.stats()
+            meta_dict[str(param_filename)] = param_stats.metadata
 
-                logger.info(f'\n\n{param_filename.name}:\n')
-                logger.info(f'Method: {method}')
-                logger.info(f'Kernel shape: {tags["HOMO_KERNEL_SHAPE"]}')
-                logger.info(f'Processing CRS: {tags["HOMO_PROC_CRS"]}')
-                if method == 'gain-offset':
-                    logger.info(f'R\N{SUPERSCRIPT TWO} inpaint threshold: {r2_inpaint_thresh}')
+        # iterate over stored result(s) and print
+        for param_filename, param_dict in stats_dict.items():
+            param_meta = meta_dict[param_filename]
 
-            cmb_dict[str(param_filename)], stats_str = utils.param_stats(param_filename, method=Method(method),
-                                                                         r2_inpaint_thresh=r2_inpaint_thresh)
-            logger.info(f'Stats:\n\n{stats_str}')
+            logger.info(f'\n{pathlib.Path(param_filename).name}:\n')
+            logger.info(param_meta)
+            # format the statistics as a dataframe to get printable string
+            param_df = pd.DataFrame.from_dict(param_dict, orient='index')
+            param_str = param_df.to_string(float_format="{:.2f}".format, index=True, justify="center",
+                                           index_names=False)
+            logger.info(f'Stats:\n{param_str}')
 
         if output is not None:
             with open(output, 'w') as file:
-                json.dump(cmb_dict, file)
+                json.dump(stats_dict, file, allow_nan=True)
 
     except Exception:
         logger.exception("Exception caught during processing")
