@@ -190,7 +190,7 @@ class RasterFuse:
                 if isinstance(other_value, dict):
                     # flatten the driver specific nested dict into the root dict
                     nested_update(self_dict, other_value)
-                elif other_value is not None:
+                else:
                     self_dict[other_key] = other_value
             return self_dict
 
@@ -210,6 +210,7 @@ class RasterFuse:
         else:
             init_profile = self._raster_pair.src_im.profile
         param_profile = self._combine_with_config(init_profile)
+        # force dtype and nodata to defaults
         param_profile.update(dtype=RasterArray.default_dtype, count=len(self._raster_pair.src_bands) * 3,
                              nodata=RasterArray.default_nodata)
         return param_profile
