@@ -165,8 +165,8 @@ class RasterPairReader:
             # warn if the proc_crs value does not correspond to the lowest resolution of the source and
             # reference images
             rec_crs_str = ProcCrs.ref if src_pixel_smaller else ProcCrs.src
-            logger.warning(f"proc_crs='{rec_crs_str}' is recommended when "
-                           f"the source image pixel size is {cmp_str} than the reference.")
+            logger.warning(f"proc_crs={rec_crs_str} is recommended when "
+                           f"the source pixel size is {cmp_str} than the reference.")
         return proc_crs
 
     def _auto_block_shape(self, proc_win: Window = None):
@@ -393,7 +393,7 @@ class RasterPairReader:
                 proc_out_block = Window(*out_ul[::-1], *np.subtract(out_br, out_ul)[::-1])
 
                 # create equivalent rasterio windows in 'other' space
-                other_in_block = utils.round_window_to_grid(other_im.window(*proc_im.window_bounds(proc_in_block)))
+                other_in_block = utils.expand_window_to_grid(other_im.window(*proc_im.window_bounds(proc_in_block)))
                 other_out_block = utils.round_window_to_grid(other_im.window(*proc_im.window_bounds(proc_out_block)))
 
                 # create the BlockPair named tuple, assigning 'proc' and 'other' back to 'src' and 'ref' for passing to
